/*
 * File: OrbitalMechanics.js
 * -------------------------
 * This program models a planet orbiting a star.
 */

/* Constants */
const GWINDOW_WIDTH = 500;
const GWINDOW_HEIGHT = 300;
const TIME_STEP = 20;
const GRAVITATIONAL_CONSTANT = 3000;
const SUN_RADIUS = 15;
const PLANET_RADIUS = 3;
const INITIAL_X_DISTANCE = 150;
const INITIAL_Y_VELOCITY = 3;

/* Main program */
function OrbitalMechanics() {
   let gw = GWindow(GWINDOW_WIDTH, GWINDOW_HEIGHT);
   let sx = gw.getWidth() / 2;
   let sy = gw.getHeight() / 2;
   let sun = createFilledCircle(sx, sy, SUN_RADIUS, "Goldenrod");
   gw.add(sun);
   let px = sx + INITIAL_X_DISTANCE;
   let py = sy;
   let planet = createFilledCircle(px, py, PLANET_RADIUS, "Blue");
   gw.add(planet);
   let vx = 0;
   let vy = INITIAL_Y_VELOCITY;
   let step = function() {
      planet.move(vx, vy);
		  px += vx;
 		  py += vy;
      let dx = sx - px;
      let dy = sy - py;
      let rSquared = dx * dx + dy * dy;
      let r = Math.sqrt(rSquared);
      let ax = GRAVITATIONAL_CONSTANT / rSquared * dx / r;
      let ay = GRAVITATIONAL_CONSTANT / rSquared * dy / r;
      vx += ax;
      vy += ay;
   };
   let timer = setInterval(step, TIME_STEP);
}

/*
 * Creates a circle of radius r centered at the point (x, y) filled
 * with the specified color.
 */

function createFilledCircle(x, y, r, color) {
   let circle = GOval(x - r, y - r, 2 * r, 2 * r);
   circle.setColor(color);
   circle.setFilled(true);
   return circle;
}
